# Sage file to create barvinok input for plethysm

def non_negativity_rows(p):
    """
    p -- a partition
    """
    l = p.length()
    n = p.size()
    n2 = p.size()-1
    I = identity_matrix(l*n2)
    Z = zero_matrix(l*n2 , n2)
    O = matrix(l*n2,1, lambda x,y: 1)
    return block_matrix(1,3,[O,I,Z])

def equality_rows(p, perm):
    """
    p -- a partition

    Ordering of rows is first all the variables, then all the column
    sums, then the row sum.
    """
    l = p.length()
    n = p.size()
    n2 = p.size()-1
    def entry(i,j):
        if floor (j/n2) == i:
            return 1
        else:
            return 0
    M = matrix (l, l*n2, entry)
    Z1 = matrix(l,1, lambda x,y: 0)
    Z2 = matrix(l,n-2, lambda x,y:0)
    O = matrix (l,1, lambda x,y:-1)
    # These are the row sum equations
    row_sums = pad_with_zero(block_matrix(1,4,[Z1,M,Z2,O]))

    def entry2(i,j):
        if mod(j,n-1) == i:
            return p[floor(j/(n-1))]
        else:
            return 0
    M = matrix (n-2, l*n2, entry2)
    O = -identity_matrix (n-2);
    Z1 = matrix(n-2,1, lambda x,y: 0)
    col_sums = block_matrix(1,4,[Z1,M,O,Z1])
    # Translation of the shift formula
    v = [-n+perm[i-1]+i for i in range(n-1,1,-1)]
    col_sums = pad_with_nonzero(col_sums, v)
    result = block_matrix(2,1,[row_sums,col_sums])
    return result

def b_inequalities(p):
    n = p.size()
    def entry (i,j):
        if i==j:
            return -1
        if i==(j-1):
            return 1
        else:
            return 0
    M = matrix (n-3,n-1, entry)
    N = matrix (1,n-1, [-1 for i in range(n-3)]+[-2,n])
    R = identity_matrix (n-1);
    O = matrix (n-2+n-1,1, lambda i,j: 1)
    return block_matrix(1,2,[O,block_matrix(3,1,[M,N,R])])

def pad_with_zero (M):
    r = len(M.rows())
    c = len(M.columns())
    return block_matrix(1,2, [M, matrix(r,1,lambda i,j:0)])

def pad_with_nonzero (M,v):
    r = len(M.rows())
    c = len(M.columns())
    return block_matrix(1,2, [M, matrix(r,1,lambda i,j:v[i])])

def barv_matrix(M):
    s = str(len(M.rows())) + " " + str(len(M.columns())) + "\n"
    for r in M.rows():
        s+=" ".join([str(c) for c in r ])
        s+="\n"
    return s

# For a partition of n we need an length of partition times n-2 array of variables:

def create_barvinok_problem (partition, permutation):
    nonneg=pad_with_zero(non_negativity_rows(partition))
    # equality_rows is padded correctly.
    poly_constraints=block_matrix(2,1,[nonneg,equality_rows(partition, permutation)])
    chamber_constraints=pad_with_zero(b_inequalities(partition))

    result = ""

    result += barv_matrix(poly_constraints)
    result += barv_matrix(chamber_constraints)

    # The extra constraints are used to represent the constant
    # polynomial on the cone, to cut away stray chambers on the
    # outside later.
    # return result + " ".join(["b"+str(i) for i in range(1,partition.size()-1)]) + " s"
    return result

def number_of_perms_with_cycle_type (a):
    return len([p for p in Permutations(a.size()) if p.cycle_type() == a])

def printPartitionSane (p):
    return "".join([str(pp) for pp in p])

def printPermutationSane (p):
    return "".join([str(pp) for pp in p])

def create_all_barvinok_problems (n):
    """
    Run through the permutations \pi and partitions \alpha and create
    barvinok input files.  Components of the filenames:
    plethysm.d.permutation.partition.barv
    """
    sumpermsstring = [];
    for perm in Permutations (n-1):
        permname = "P"+printPermutationSane(perm)
        sumpermsstring = sumpermsstring + [permname]
        i = 0;
        for p in Partitions(n):
            filename = ".".join(["plethysm",
                                 str(n),
                                 printPermutationSane(perm),
                                 printPartitionSane(p),
                                 "barv"])
            f = open(filename,'w')
            i += 1;
            s = create_barvinok_problem(p, perm)
            f.write(s)
            f.write(" ".join(["b"+str(i) for i in range(1,n-1)]) + " s")



def create_iscc_summation (n, mu_coefficients, filename):
    f = open(filename, 'w')
    # Gather names of permutations for hierarchical summing in n-1 lists
    sumpermsstrings = [[],[],[],[]]
    for perm in Permutations (n-1):
        permname = "P"+printPermutationSane(perm)
        if perm[0]==1
        sumpermsstring = sumpermsstring + [permname]
        sumpartstrings = [];
        i = 0;
        for p in Partitions(n):
            probfile = ".".join(["plethysm",
                                 str(n),
                                 printPermutationSane(perm),
                                 printPartitionSane(p),
                                 "barv"])
            sumpart = "(" + " * ".join([str(mu_coefficients[i]),
                                        str(number_of_perms_with_cycle_type(p)),
                                        str(perm.sign()),
                                        " read \"" ]) + probfile + ".result\")"
            i += 1;
            sumpartstrings += [sumpart]
        f.write (permname + ":=" + "+".join(sumpartstrings) + ";\n")
    f.write ( "write \"final.result\" " + "+".join(sumpermsstring) + ";\n")
    f.close()

